#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <windows.h>
#include <WINIOCTL.H>
#include <objbase.h>
#include <initguid.h>
#include "../drv/I2cBridgeDriverInterface.h"
#include "dev_rout.h"
#include "../common/i2cbridge.h"

#define MAX_USB_DEV_NUMBER 8
#define MAX_DRIVER_NAME 64
#define DEVICE_NAME_TEMPLATE "\\\\.\\UsbI2CIo-%d"


BOOL APIENTRY DllMain( HANDLE, 
                       DWORD, 
                       LPVOID
					 )
{
    return TRUE;
}


BYTE dr_get_device_count()
{
	char DeviceName[MAX_DRIVER_NAME] = "";
	HANDLE hDevice;
	BYTE DeviceNum = 0;
	BYTE i;
	for (i=0; i<MAX_USB_DEV_NUMBER; i++)
	{
		sprintf(DeviceName, DEVICE_NAME_TEMPLATE, i);
		hDevice = CreateFile(DeviceName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
		NULL, OPEN_EXISTING, 0, NULL);
		if (hDevice != INVALID_HANDLE_VALUE)
		{
			CloseHandle(hDevice);
			DeviceNum++;
		}
	}
	return DeviceNum;
}

HANDLE dr_open_device(BYTE nDevice)
{
	char DeviceName[MAX_DRIVER_NAME] = "";
	HANDLE hDevice = INVALID_HANDLE_VALUE;
	BYTE DeviceNum = 0;
	BYTE i;
	for (i=0; i<MAX_USB_DEV_NUMBER; i++)
	{
		sprintf(DeviceName, DEVICE_NAME_TEMPLATE, i);
		hDevice = CreateFile(DeviceName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
			NULL, OPEN_EXISTING, 0, NULL);
		if (hDevice != INVALID_HANDLE_VALUE)
		{
			if (DeviceNum == nDevice)
				break;
			else
			{
				CloseHandle(hDevice);
				DeviceNum++;
			}
		}
	}
	return hDevice;
}


U2C_RESULT dr_close_device(HANDLE hDevice)
{
	if(CloseHandle(hDevice))
		return U2C_SUCCESS;
	else
		return U2C_HARDWARE_NOT_FOUND;
}


U2C_RESULT dr_is_handle_valid(HANDLE hDevice)
{
	U2C_VERSION_INFO ver;
	if (dr_get_driver_version(hDevice, &ver) == U2C_SUCCESS)
		return U2C_SUCCESS;
	else
		return U2C_HARDWARE_NOT_FOUND;
}






U2C_RESULT dr_get_driver_version(HANDLE hDevice, PU2C_VERSION_INFO pVersion)
{
	ULONG Returned;
	DeviceIoControl(hDevice, IOCTL_GET_DRIVER_VER, pVersion, sizeof(DRIVER_VERSION),
		pVersion, sizeof(U2C_VERSION_INFO), &Returned, NULL);
	if (Returned != sizeof(U2C_VERSION_INFO))
		return U2C_UNKNOWN_ERROR;
	return U2C_SUCCESS;
}

U2C_RESULT u2c_fw_rc( int rc )
{
    if( rc )
        switch( rc )
        {            
        case 1:
            return U2C_TRANSACTION_FAILED;
        case 2:
            return U2C_SLAVE_OPENNING_FOR_WRITE_FAILED;
        case 3:
            return U2C_SLAVE_OPENNING_FOR_READ_FAILED;
        case 4:
            return U2C_SENDING_MEMORY_ADDRESS_FAILED;
        case 5:
            return U2C_SENDING_DATA_FAILED;
        case 6:
            return U2C_NOT_IMPLEMENTED;
        case 7:
            return U2C_NO_ACK;    
		case 8:
			return U2C_UNKNOWN_ERROR;
		case 9:
			return U2C_I2C_CLOCK_SYNCH_TIMEOUT;
        default:
            return U2C_UNKNOWN_ERROR;
        }
    else
        return U2C_SUCCESS;
}   


U2C_RESULT dr_u2c_transaction(HANDLE hDevice, PDEVICE_REQUEST req)
{
	ULONG Returned;
	ULONG in_buf_len = req->InputBufferLength;
	BOOL  bResult = DeviceIoControl(hDevice, IOCTL_DEVICE_REQUEST, req, sizeof(DEVICE_REQUEST),
		req, sizeof(DEVICE_REQUEST), &Returned, NULL);
	if (!bResult)
		return U2C_HARDWARE_NOT_FOUND;
	if (in_buf_len == 0)
		in_buf_len = req->InputBufferLength;
	if(( in_buf_len != req->InputBufferLength ) || (req->InputBufferLength == 0))
        return U2C_TRANSACTION_FAILED;

    /* Check Status. Status is returned by fw in last byte */
    if( req->InputBuffer[req->InputBufferLength-1] )
        return u2c_fw_rc( req->InputBuffer[req->InputBufferLength-1] );
    else
        return U2C_SUCCESS;
}
