/***************************************************************************
 *   Copyright (C)2004 Diolan ( http://www.diolan.com )                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
*-------------------------------------------------------------------
* Description: i2cbrdg main module
*-------------------------------------------------------------------
* Version: $Id: i2cbridge.c,v 1.2 2005/11/05 13:59:42 xol Exp $
*-------------------------------------------------------------------
*/
#ifdef WIN32
#include <windows.h>
#endif
#include <stdlib.h>

#include "string.h"
#include "../common/i2cbridge.h"
#include "dev_rout.h"

#define maj_ver 1
#define min_ver 0



//-----------------------------------------------------------------------------
// Open/Close Functions
//-----------------------------------------------------------------------------
BYTE _stdcall U2C_GetDeviceCount( void )
{
	return dr_get_device_count();
}


HANDLE _stdcall U2C_OpenDevice(BYTE nDevice)
{
	return dr_open_device(nDevice);
}


U2C_RESULT _stdcall U2C_CloseDevice(HANDLE hDevice)
{
	return dr_close_device(hDevice);
}


U2C_RESULT _stdcall U2C_IsHandleValid(HANDLE hDevice)
{
	return dr_is_handle_valid(hDevice);
}


U2C_RESULT _stdcall U2C_GetSerialNum(HANDLE hDevice, long* pSerialNum)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 1;
	req.OutputBuffer[0] = 0x0B;
	req.InputBufferLength = 5;

	rc = dr_u2c_transaction( hDevice, &req );
	memcpy(pSerialNum, req.InputBuffer, 4);
	return rc;
}


U2C_RESULT _stdcall U2C_GetFirmwareVersion(HANDLE hDevice, PU2C_VERSION_INFO pVersion)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 1;
	req.OutputBuffer[0] = 0x0A;
	req.InputBufferLength = 3;

	rc = dr_u2c_transaction( hDevice, &req );
	pVersion->MajorVersion = req.InputBuffer[0];
	pVersion->MinorVersion = req.InputBuffer[1];
	return rc;
}

HANDLE _stdcall U2C_OpenDeviceBySerialNum(long nSerialNum)
{
	BYTE dev_num, i;
	HANDLE dev;
	long sn;
	dev_num = U2C_GetDeviceCount();
	for (i = 0; i < dev_num; i++)
	{
		dev = U2C_OpenDevice(i);
		if ((dev != INVALID_HANDLE_VALUE)
			&& (U2C_SUCCESS == U2C_GetSerialNum(dev, &sn))
			&& (sn == nSerialNum))
		{
			return dev;
		}
		U2C_CloseDevice(dev);
	}
	return INVALID_HANDLE_VALUE;
}

U2C_VERSION_INFO _stdcall U2C_GetDllVersion()
{
	U2C_VERSION_INFO ver;
	ver.MajorVersion = maj_ver;
	ver.MinorVersion = min_ver;
	return ver;
}

U2C_RESULT _stdcall U2C_GetDriverVersion(HANDLE hDevice, PU2C_VERSION_INFO pVersion)
{
	return dr_get_driver_version(hDevice, pVersion);
}


//-----------------------------------------------------------------------------
// I2C high level and configuration routines
//-----------------------------------------------------------------------------

// Convert I2C Transaction into U2C device request
static U2C_RESULT _stdcall u2c_prep_i2c_req( PU2C_TRANSACTION pTransaction, PDEVICE_REQUEST preq, BOOL rw )
{
	int     i=0;

	if (pTransaction->nSlaveDeviceAddress > 0x7F)
		return U2C_BAD_PARAMETER;
	if( rw )
		preq->OutputBuffer[i++] = 0x01;     // Read request
	else
		preq->OutputBuffer[i++] = 0x02;     // Write request

	// Memory address size
	preq->OutputBuffer[i++] = pTransaction->nMemoryAddressLength;

	// Write slave address
	if( !rw || pTransaction->nMemoryAddressLength )
		preq->OutputBuffer[i++] = (pTransaction->nSlaveDeviceAddress<<1) & 0xFE;

	// Memory Address. LSB first
	if(pTransaction->nMemoryAddressLength > 4) 
		return U2C_BAD_PARAMETER;
	if( pTransaction->nMemoryAddressLength == 4 )
		preq->OutputBuffer[i++] = (UCHAR)((pTransaction->nMemoryAddress>>24) & 0xFF);
	if( pTransaction->nMemoryAddressLength >= 3 )
		preq->OutputBuffer[i++] = (UCHAR)((pTransaction->nMemoryAddress>>16) & 0xFF);
	if( pTransaction->nMemoryAddressLength >= 2 )
		preq->OutputBuffer[i++] = (UCHAR)((pTransaction->nMemoryAddress>>8) & 0xFF);
	if( pTransaction->nMemoryAddressLength >= 1 )
		preq->OutputBuffer[i++] = (UCHAR)(pTransaction->nMemoryAddress & 0xFF);

	// Read Slave address
	if( rw )
		preq->OutputBuffer[i++] = (pTransaction->nSlaveDeviceAddress<<1) | 0x01;

	// Data size
	if ((pTransaction->nBufferLength > 256) || (pTransaction->nBufferLength == 0))
		return U2C_BAD_PARAMETER;

	if (pTransaction->nBufferLength == 256)
		preq->OutputBuffer[i++] = 0;
	else
		preq->OutputBuffer[i++] = (UCHAR)pTransaction->nBufferLength;

	// Data to write
	if( !rw )
	{
		memcpy( &preq->OutputBuffer[i], pTransaction->Buffer, pTransaction->nBufferLength );
		i+= pTransaction->nBufferLength;
	}

	preq->OutputBufferLength = i;
	if( rw )
		preq->InputBufferLength = pTransaction->nBufferLength+1;
	else
		preq->InputBufferLength = 1;
	return U2C_SUCCESS;
}

U2C_RESULT _stdcall U2C_Read(HANDLE hDevice, PU2C_TRANSACTION pTransaction)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	rc = u2c_prep_i2c_req( pTransaction, &req, 1 );
	if( U2C_SUCCESS != rc ) return rc;

	rc = dr_u2c_transaction( hDevice, &req );
	if( rc ) return rc;

	memcpy(pTransaction->Buffer, req.InputBuffer, pTransaction->nBufferLength);
	return rc;
}

U2C_RESULT _stdcall U2C_Write(HANDLE hDevice, PU2C_TRANSACTION pTransaction)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	rc = u2c_prep_i2c_req( pTransaction, &req, 0 );
	if( U2C_SUCCESS != rc ) return rc;

	return dr_u2c_transaction( hDevice, &req );
}

#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__CYGWIN__)
U2C_RESULT _stdcall U2C_RW_Pack(HANDLE hDevice, PU2C_TRANSACTION_PACK pTransaction, int count)
{
	U2C_RESULT rc;
	PDEVICE_REQUEST preq;
	int     i;

	preq = malloc( count * sizeof(DEVICE_REQUEST) );

	// Prepare request pack
	for( i=0; i<count; i++ )
	{
		rc = u2c_prep_i2c_req( &pTransaction[i].tr, &preq[i], pTransaction[i].rw );
		if( U2C_SUCCESS != rc ) return rc;
	}

	// Execute
	rc = dr_u2c_transaction_pack( hDevice, preq, count );

	// Extract results
	if( !rc )
	{
		for( i=0; i<count; i++ )
		{
			// Check reply size
			if( (pTransaction[i].rw  && (preq[i].InputBufferLength!=pTransaction[i].tr.nBufferLength+1)) ||
				(!pTransaction[i].rw && (preq[i].InputBufferLength!=1)) )
			{
				pTransaction[i].rc = U2C_TRANSACTION_FAILED;
				continue;
			}

			// Check status
			if( preq[i].InputBuffer[preq[i].InputBufferLength-1] )
				pTransaction[i].rc = u2c_fw_rc( preq[i].InputBuffer[preq[i].InputBufferLength-1] );
			else
				pTransaction[i].rc = U2C_SUCCESS;

			// Take data
			memcpy( pTransaction[i].tr.Buffer, preq[i].InputBuffer, preq[i].InputBufferLength-1 );
		}
	}
	free(preq);
	return rc;
}
#endif /* defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__CYGWIN__) */

U2C_RESULT _stdcall U2C_SetI2cFreq(HANDLE hDevice, BYTE Frequency)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 2;
	req.OutputBuffer[0] = 0x1B;
	req.OutputBuffer[1] = Frequency;
	req.InputBufferLength = 1;

	rc = dr_u2c_transaction( hDevice, &req );
	return rc;
}

U2C_RESULT _stdcall U2C_GetI2cFreq(HANDLE hDevice, BYTE *pFrequency)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 1;
	req.OutputBuffer[0] = 0x1C;
	req.InputBufferLength = 2;

	rc = dr_u2c_transaction(hDevice, &req);
	*pFrequency = req.InputBuffer[0];
	return rc;
}

U2C_RESULT _stdcall U2C_SetClockSynch(HANDLE hDevice, BOOL Enable)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 2;
	req.OutputBuffer[0] = 0x24;
	req.OutputBuffer[1] = (BYTE)Enable;
	req.InputBufferLength = 1;

	rc = dr_u2c_transaction( hDevice, &req );
	return rc;
}

U2C_RESULT _stdcall U2C_GetClockSynch(HANDLE hDevice, BOOL *pEnable)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 1;
	req.OutputBuffer[0] = 0x25;
	req.InputBufferLength = 2;

	rc = dr_u2c_transaction(hDevice, &req);
	*pEnable = req.InputBuffer[0];
	return rc;
}

U2C_RESULT _stdcall U2C_SetClockSynchTimeout(HANDLE hDevice, WORD Timeout)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;
	req.OutputBufferLength = 3;
	req.OutputBuffer[0] = 0x26;
	req.OutputBuffer[1] = (UCHAR)(Timeout & 0xFF);
	req.OutputBuffer[2] = (UCHAR)((Timeout>>8) & 0xFF);
	req.InputBufferLength = 1;

	rc = dr_u2c_transaction(hDevice, &req);
	return rc;
}

U2C_RESULT _stdcall U2C_GetClockSynchTimeout(HANDLE hDevice, WORD *pTimeout)
{
	U2C_RESULT rc;
	DEVICE_REQUEST req;

	req.OutputBufferLength = 1;
	req.OutputBuffer[0] = 0x27;
	req.InputBufferLength = 3;

	rc = dr_u2c_transaction(hDevice, &req);
	*pTimeout = (req.InputBuffer[1]<<8)|(req.InputBuffer[0]);
	return rc;
}


U2C_RESULT _stdcall U2C_ScanDevices(HANDLE hDevice, PU2C_SLAVE_ADDR_LIST pList)
{
    U2C_RESULT rc;
    DEVICE_REQUEST req;
    ULONG i;
	pList->nDeviceNumber = 0;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x3;
    req.InputBufferLength = 0;
    if (U2C_SUCCESS != (rc = dr_u2c_transaction(hDevice, &req)))
		return rc;
    for (i = 0; i < req.InputBufferLength - 1; i++)
       pList->List[i] = (req.InputBuffer[i] >> 1);
    pList->nDeviceNumber = (BYTE)(req.InputBufferLength - 1);
    return rc;
}



//-----------------------------------------------------------------------------
// I2C low level routines
//-----------------------------------------------------------------------------

U2C_RESULT _stdcall U2C_Start(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x0C;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_RepeatedStart(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x0E;
    req.InputBufferLength= 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_Stop(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x0D;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_PutByte(HANDLE hDevice, BYTE Data)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x0F;
    req.OutputBuffer[1] = Data;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_GetByte(HANDLE hDevice, BYTE *pData)
{
    U2C_RESULT rc;
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x10;
    req.InputBufferLength = 2;
    rc = dr_u2c_transaction(hDevice, &req);
    *pData = req.InputBuffer[0];
    return rc;
}

U2C_RESULT _stdcall U2C_PutByteWithAck(HANDLE hDevice, BYTE Data)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x13;
    req.OutputBuffer[1] = Data;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_GetByteWithAck(HANDLE hDevice, BYTE *pData, BOOL bAck)
{
    U2C_RESULT rc;
    DEVICE_REQUEST req;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x14;
    req.OutputBuffer[1] = (UCHAR)bAck;
    req.InputBufferLength = 2;
    rc = dr_u2c_transaction(hDevice, &req);
    *pData = req.InputBuffer[0];
    return rc;
}

U2C_RESULT _stdcall U2C_PutAck(HANDLE hDevice, BOOL bAck)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x11;
    req.OutputBuffer[1] = (UCHAR)bAck;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_GetAck(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x12;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}



//-----------------------------------------------------------------------------
// I2C wire level routines
//-----------------------------------------------------------------------------
static U2C_RESULT read_line_state(HANDLE hDevice, U2C_LINE_STATE *pState, BYTE command)
{
    U2C_RESULT rc;
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = command;
    req.InputBufferLength = 2;
    rc = dr_u2c_transaction(hDevice, &req);
    switch (req.InputBuffer[0])
    {
    case 0xC0:
       *pState = LS_RELEASED;
       break;
    case 0x00:
       *pState = LS_DROPPED_BY_I2C_BRIDGE;
       break;
    case 0x40:
       *pState = LS_DROPPED_BY_SLAVE;
       break;
    case 0x80:
       *pState = LS_RAISED_BY_I2C_BRIDGE;
       break;
    default:
       if (rc == U2C_SUCCESS)
          rc = U2C_UNKNOWN_ERROR;
    }
    return rc;
}

U2C_RESULT _stdcall U2C_ReadScl(HANDLE hDevice, U2C_LINE_STATE *pState)
{
    return read_line_state(hDevice, pState, 0x09);
}

U2C_RESULT _stdcall U2C_ReadSda(HANDLE hDevice, U2C_LINE_STATE *pState)
{
    return read_line_state(hDevice, pState, 0x08);
}

U2C_RESULT _stdcall U2C_ReleaseScl(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x05;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_ReleaseSda(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x04;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_DropScl(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x07;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_DropSda(HANDLE hDevice)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x06;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}



//-----------------------------------------------------------------------------
// GPIO routines
//-----------------------------------------------------------------------------
U2C_RESULT _stdcall U2C_SetIoDirection(HANDLE hDevice, ULONG Value, ULONG Mask)
{
    DEVICE_REQUEST req;
    Value = Value & Mask;
    Mask = ~Mask;
    req.OutputBufferLength = 7;
    req.OutputBuffer[0] = 0x17;
    req.OutputBuffer[1] = (BYTE)Mask;
    req.OutputBuffer[2] = (BYTE)Value;
    req.OutputBuffer[3] = (BYTE)(Mask >> 8);
    req.OutputBuffer[4] = (BYTE)(Value >> 8);
    req.OutputBuffer[5] = (BYTE)(Mask >> 16);
    req.OutputBuffer[6] = (BYTE)(Value >> 16);
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}


U2C_RESULT _stdcall U2C_GetIoDirection(HANDLE hDevice, ULONG *pValue)
{
    U2C_RESULT rc;
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x18;
    req.InputBufferLength = 4;
    rc = dr_u2c_transaction(hDevice, &req);
    *pValue = req.InputBuffer[0];
    (*pValue) |= (req.InputBuffer[1] << 8);
    (*pValue) |= (req.InputBuffer[2] << 16);
    return rc;
}



U2C_RESULT _stdcall U2C_IoWrite(HANDLE hDevice, ULONG Value, ULONG Mask)
{
    DEVICE_REQUEST req;
    Value = Value & Mask;
    Mask = ~Mask;
    req.OutputBufferLength = 7;
    req.OutputBuffer[0] = 0x19;
    req.OutputBuffer[1] = (BYTE)Mask;
    req.OutputBuffer[2] = (BYTE)Value;
    req.OutputBuffer[3] = (BYTE)(Mask >> 8);
    req.OutputBuffer[4] = (BYTE)(Value >> 8);
    req.OutputBuffer[5] = (BYTE)(Mask >> 16);
    req.OutputBuffer[6] = (BYTE)(Value >> 16);
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_IoRead(HANDLE hDevice, ULONG *pValue)
{
    U2C_RESULT rc;
    DEVICE_REQUEST req;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x1A;
    req.InputBufferLength = 4;
    rc = dr_u2c_transaction(hDevice, &req);
    *pValue = req.InputBuffer[0];
    (*pValue) |= (req.InputBuffer[1] << 8);
    (*pValue) |= (req.InputBuffer[2] << 16);
    return rc;
}

U2C_RESULT _stdcall U2C_SetSingleIoDirection(HANDLE hDevice, ULONG IoNumber, BOOL bOutput)
{
    ULONG Value = 0;
    ULONG Mask = 1;
    if (IoNumber >= 8*3)
       return U2C_BAD_PARAMETER;
    Mask <<= IoNumber;
    if (bOutput)
       Value = Mask;
    return U2C_SetIoDirection(hDevice, Value, Mask);
}

U2C_RESULT _stdcall U2C_GetSingleIoDirection(HANDLE hDevice, ULONG IoNumber, BOOL *pbOutput)
{
    U2C_RESULT rc;
    ULONG Value;
    ULONG Mask = 1;
    if (IoNumber >= 8*3)
       return U2C_BAD_PARAMETER;
    rc = U2C_GetIoDirection(hDevice, &Value);
    Mask <<= IoNumber;
    if ((Value & Mask) == 0)
      *pbOutput = FALSE;
    else
      *pbOutput = TRUE;
    return rc;
}

U2C_RESULT _stdcall U2C_SingleIoWrite(HANDLE hDevice, ULONG IoNumber, BOOL Value)
{
    ULONG val = 0;
    ULONG mask = 1;
    if (IoNumber >= 8*3)
       return U2C_BAD_PARAMETER;
    mask <<= IoNumber;
    if (Value)
       val = mask;
    return U2C_IoWrite(hDevice, val, mask);
}

U2C_RESULT _stdcall U2C_SingleIoRead(HANDLE hDevice, ULONG IoNumber, BOOL *pValue)
{
    U2C_RESULT rc;
    ULONG val;
    ULONG mask = 1;
    if (IoNumber >= 8*3)
       return U2C_BAD_PARAMETER;
    rc = U2C_IoRead(hDevice, &val);
    mask <<= IoNumber;
    if ((val & mask) == 0)
      *pValue = FALSE;
    else
      *pValue = TRUE;
    return rc;
}


//-----------------------------------------------------------------------------
// SPI configuration routines
//-----------------------------------------------------------------------------

U2C_RESULT _stdcall U2C_SpiSetConfigEx(HANDLE hDevice, DWORD Config)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x1D;
    req.OutputBuffer[1] = (UCHAR)Config;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_SpiGetConfigEx(HANDLE hDevice, DWORD *pConfig)
{
    DEVICE_REQUEST req;
    U2C_RESULT rc;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x1E;
    req.InputBufferLength = 2;
    rc = dr_u2c_transaction(hDevice, &req);
	*pConfig = req.InputBuffer[0];
    return rc;
}

U2C_RESULT _stdcall U2C_SpiSetConfig(HANDLE hDevice, BYTE CPOL, BYTE CPHA)
{
	DWORD config = 0x00;
	if (CPOL)
		config |= 0x1;
	if (CPHA)
		config |= 0x2;
	return U2C_SpiSetConfigEx(hDevice, config);
}

U2C_RESULT _stdcall U2C_SpiGetConfig(HANDLE hDevice, BYTE *pCPOL, BYTE *pCPHA)
{
	DWORD config;
	U2C_RESULT rc = U2C_SpiGetConfigEx(hDevice, &config);
	if (config & 0x1)
		*pCPOL = 1;
	else
		*pCPOL = 0;
	if (config & 0x2)
		*pCPHA = 1;
	else
		*pCPHA = 0;
	return rc;
}



U2C_RESULT _stdcall U2C_SpiSetFreq(HANDLE hDevice, BYTE Frequency)
{
    DEVICE_REQUEST req;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x1F;
    req.OutputBuffer[1] = Frequency;
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_SpiGetFreq(HANDLE hDevice, BYTE *pFrequency)
{
    DEVICE_REQUEST req;
    U2C_RESULT rc;
    req.OutputBufferLength = 1;
    req.OutputBuffer[0] = 0x20;
    req.InputBufferLength = 2;
    rc = dr_u2c_transaction(hDevice, &req);
    *pFrequency = req.InputBuffer[0];
    return rc;
}

//-----------------------------------------------------------------------------
// SPI data transfer routines
//-----------------------------------------------------------------------------
U2C_RESULT _stdcall U2C_SpiReadWrite(HANDLE hDevice, BYTE *pOutBuffer, BYTE *pInBuffer, WORD Length)
{
    DEVICE_REQUEST req;
    U2C_RESULT rc;
    if ((Length == 0) || (Length > 256))
       return U2C_BAD_PARAMETER;
    req.OutputBufferLength = Length + 2;
    req.OutputBuffer[0] = 0x21;
    req.OutputBuffer[1] = (BYTE)Length;
    memcpy(req.OutputBuffer+2, pOutBuffer, Length);
    req.InputBufferLength = Length + 1;
    rc = dr_u2c_transaction(hDevice, &req);
    memcpy(pInBuffer, req.InputBuffer, Length);
    return rc;
}

U2C_RESULT _stdcall U2C_SpiWrite(HANDLE hDevice, BYTE *pOutBuffer, WORD Length)
{
    DEVICE_REQUEST req;
    if ((Length == 0) || (Length > 256))
       return U2C_BAD_PARAMETER;
    req.OutputBufferLength = Length + 2;
    req.OutputBuffer[0] = 0x23;
    req.OutputBuffer[1] = (BYTE)Length;
    memcpy(req.OutputBuffer+2, pOutBuffer, Length);
    req.InputBufferLength = 1;
    return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_SpiRead(HANDLE hDevice, BYTE *pInBuffer, WORD Length)
{
    DEVICE_REQUEST req;
    U2C_RESULT rc;
    if ((Length == 0) || (Length > 256))
       return U2C_BAD_PARAMETER;
    req.OutputBufferLength = 2;
    req.OutputBuffer[0] = 0x22;
    req.OutputBuffer[1] = (BYTE)Length;
    req.InputBufferLength = Length + 1;
    rc = dr_u2c_transaction(hDevice, &req);
    memcpy(pInBuffer, req.InputBuffer, Length);
    return rc;
}

U2C_RESULT _stdcall U2C_SpiReadWriteSS(HANDLE hDevice, BYTE *pOutBuffer, BYTE *pInBuffer, WORD Length,
									   ULONG IoNumber, BOOL ActiveHigh)
{
	DEVICE_REQUEST req;
	U2C_RESULT rc;
	if ((Length == 0) || (Length > 256) || (IoNumber >= 8*3))
		return U2C_BAD_PARAMETER;
	req.OutputBufferLength = Length + 2 + 3;
	req.OutputBuffer[0] = 0x28;
	req.OutputBuffer[1] = ActiveHigh ? 1 : 0;
	req.OutputBuffer[2] = (UCHAR)(IoNumber/8);
	UCHAR ss_pin = 1;
	ss_pin <<= (IoNumber%8);
	req.OutputBuffer[3] = ss_pin;
	req.OutputBuffer[4] = (BYTE)Length;
	memcpy(req.OutputBuffer+2+3, pOutBuffer, Length);
	req.InputBufferLength = Length + 1;
	rc = dr_u2c_transaction(hDevice, &req);
	memcpy(pInBuffer, req.InputBuffer, Length);
	return rc;
}

U2C_RESULT _stdcall U2C_SpiWriteSS(HANDLE hDevice, BYTE *pOutBuffer, WORD Length,
								   ULONG IoNumber, BOOL ActiveHigh)
{
	DEVICE_REQUEST req;
	if ((Length == 0) || (Length > 256) || (IoNumber >= 8*3))
		return U2C_BAD_PARAMETER;
	req.OutputBufferLength = Length + 2 + 3;
	req.OutputBuffer[0] = 0x2A;
	req.OutputBuffer[1] = ActiveHigh ? 1 : 0;
	req.OutputBuffer[2] = (UCHAR)(IoNumber/8);
	UCHAR ss_pin = 1;
	ss_pin <<= (IoNumber%8);
	req.OutputBuffer[3] = ss_pin;
	req.OutputBuffer[4] = (BYTE)Length;
	memcpy(req.OutputBuffer+2 + 3, pOutBuffer, Length);
	req.InputBufferLength = 1;
	return dr_u2c_transaction(hDevice, &req);
}

U2C_RESULT _stdcall U2C_SpiReadSS(HANDLE hDevice, BYTE *pInBuffer, WORD Length,
								  ULONG IoNumber, BOOL ActiveHigh)
{
	DEVICE_REQUEST req;
	U2C_RESULT rc;
	if ((Length == 0) || (Length > 256) || (IoNumber >= 8*3))
		return U2C_BAD_PARAMETER;
	req.OutputBufferLength = 2 + 3;
	req.OutputBuffer[0] = 0x29;
	req.OutputBuffer[1] = ActiveHigh ? 1 : 0;
	req.OutputBuffer[2] = (UCHAR)(IoNumber/8);
	UCHAR ss_pin = 1;
	ss_pin <<= (IoNumber%8);
	req.OutputBuffer[3] = ss_pin;
	req.OutputBuffer[4] = (BYTE)Length;
	req.InputBufferLength = Length + 1;
	rc = dr_u2c_transaction(hDevice, &req);
	memcpy(pInBuffer, req.InputBuffer, Length);
	return rc;
}

U2C_RESULT _stdcall U2C_SpiConfigSS(HANDLE hDevice, ULONG IoNumber, BOOL ActiveHigh)
{
	U2C_RESULT rc;
	if (U2C_SUCCESS != (rc = U2C_SingleIoWrite(hDevice, IoNumber, !ActiveHigh)))
		return rc;
	if (U2C_SUCCESS != (rc = U2C_SetSingleIoDirection(hDevice, IoNumber, TRUE)))
		return rc;
	return rc;
}
