/*
 Copyright (C)2004 Diolan ( http://www.diolan.com )

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
*/
  
/*
*-------------------------------------------------------------------
* Description: functional tester for i2cbrdg library 
*-------------------------------------------------------------------								 
* Version: $Id: func_test.c,v 1.1 2005/03/08 16:08:10 dmitry_kulik Exp $									   
*-------------------------------------------------------------------
*/
#include <stdio.h>
#include <i2cbridge.h>

//-----------------------------------------------------------------------------
// Constants
//-----------------------------------------------------------------------------
#define I2C_PACK_COUNT 10
#define TEST_SA 0x51

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Local Function Definition
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// MAIN
//-----------------------------------------------------------------------------
int main( int argc, char* argv[] )
{
    int     rc=0;
    HANDLE  hDevice;
    long    sn;
    U2C_VERSION_INFO fwVersion;
    BYTE    freq, prev_freq;
    U2C_SLAVE_ADDR_LIST slave_list;
    U2C_TRANSACTION trans;
    U2C_TRANSACTION_PACK trans_pack[I2C_PACK_COUNT];
    int i,j;
    BYTE Data;

    // Enumerate 
    rc = U2C_GetDeviceCount();
    printf( "U2C_GetDeviceCount: found %d Diolan u2c-12 device(s)\n", rc );
    if( rc == 0 )
        return -1;

    // Open device '0' 
    hDevice = U2C_OpenDevice( 0 );
    if( hDevice == INVALID_HANDLE_VALUE )
       return -1; 
    printf( "U2C_OpenDevice(0): hDevice=%p\n", hDevice );        

    // Get Serial Number
    rc = U2C_GetSerialNum( hDevice, &sn );
    if( rc ) return rc;
    printf( "U2C_GetSerialNum: %08lx\n", sn );

    // Get Library Version
    fwVersion = U2C_GetDllVersion();
    printf( "U2C_GetDllVersion: Major=%02hd Minor=%02hd\n", 
                            fwVersion.MajorVersion, fwVersion.MinorVersion );

    // Get FW Version
    rc = U2C_GetFirmwareVersion( hDevice, &fwVersion);
    if( rc ) return rc;
    printf( "U2C_GetFirmwareVersion: Major=%02hd Minor=%02hd\n", 
                            fwVersion.MajorVersion, fwVersion.MinorVersion );

    // I2C slaves
    rc = U2C_ScanDevices(hDevice, &slave_list);
    if ( rc ) return rc;
    printf("Found %d I2C slave devices:\n   ", slave_list.nDeviceNumber);
    for (i=0; i<slave_list.nDeviceNumber; i++)
       printf("%02X ", slave_list.List[i]);
    printf("\n");
    if( !slave_list.nDeviceNumber )
        goto skip_i2c;
   
    // I2C write/read
    trans.nSlaveDeviceAddress = TEST_SA;
    trans.nMemoryAddressLength = 2;
    trans.nMemoryAddress = 0x20;
    trans.nBufferLength = 10;
    rc = U2C_Read(hDevice, &trans);
    if (rc) return rc;
    printf("Reading 10 bytes from I2C slave 0x%02x, addr = 0x20:\n   ", TEST_SA);
    for (i=0; i<10; i++)
       printf("%02X ", trans.Buffer[i]);
    printf("\n");


    // I2C low level functions
    if (U2C_SUCCESS != (rc=U2C_Start(hDevice)))
       return rc;
    if (U2C_SUCCESS != (rc=U2C_PutByteWithAck(hDevice, TEST_SA<<1 )))
    {
       printf("rc = %d\n", rc);
       U2C_Stop(hDevice);
       return rc;
    }
    else
       printf("U2C_PutByteWithAck succeeded\n");
    if (U2C_SUCCESS != (rc=U2C_PutByte(hDevice, 0)))
       return rc;
    if (U2C_SUCCESS != (rc=U2C_GetAck(hDevice)))
       return rc;
    if (U2C_SUCCESS != (rc=U2C_PutByteWithAck(hDevice, 0x20)))
       return rc;
    if (U2C_SUCCESS != (rc=U2C_RepeatedStart(hDevice)))
       return rc;
    if (U2C_SUCCESS != (rc=U2C_PutByteWithAck(hDevice, (TEST_SA<<1)+1 )))
       return rc;
    for (i = 0; i < 10; i++)
    {
       if (U2C_SUCCESS != (rc=U2C_GetByte(hDevice, &Data)))
          return rc;
       else
          printf("%02X ", Data);
       if (U2C_SUCCESS != (rc=U2C_PutAck(hDevice, 0)))
          return rc;
    }
    if (U2C_SUCCESS != (rc=U2C_GetByteWithAck(hDevice, &Data, 0)))
      return rc;
    else
      printf("%02X ", Data);
    printf("\n");
    U2C_Stop(hDevice);
    
    // I2C Packed Transactions
    for( i=0; i<I2C_PACK_COUNT; i++ )
    {
        trans_pack[i].rw = 1;
        trans_pack[i].tr.nSlaveDeviceAddress = TEST_SA;
        trans_pack[i].tr.nMemoryAddressLength = 2;
        trans_pack[i].tr.nMemoryAddress = 16*i;
        trans_pack[i].tr.nBufferLength = 16;
    }
    rc = U2C_RW_Pack( hDevice, trans_pack, I2C_PACK_COUNT );
    if( rc ) return rc;
    for( i=0; i<I2C_PACK_COUNT; i++ )
    {        
        printf( "Transaction %d, rc=%d, in_size=%d", i,trans_pack[i].rc, trans_pack[i].tr.nBufferLength );
        if( U2C_SUCCESS == trans_pack[i].rc )
        {
            printf(" data=[ " );            
            for( j=0; j<trans_pack[i].tr.nBufferLength; j++ )
                printf( "%02x ", trans_pack[i].tr.Buffer[j] );
            printf( "]" );
        }            
        printf( "\n" );
    }        
        
skip_i2c:
    // I2C Frequency
    rc = U2C_GetI2cFreq( hDevice, &freq );
    if( rc ) return rc;
    printf( "U2C_GetI2cFreq: Frequency=%d\n", freq );
    prev_freq=freq;
    
    rc = U2C_SetI2cFreq( hDevice, U2C_I2C_FREQ_5KHZ );
    printf( "U2C_SetI2cFreq: Freq set to %d\n", U2C_I2C_FREQ_5KHZ );
    
    rc = U2C_GetI2cFreq( hDevice, &freq );
    if( rc ) return rc;
    printf( "U2C_GetI2cFreq: Frequency=%d\n", freq );
    
    rc = U2C_SetI2cFreq( hDevice, prev_freq );
    printf( "U2C_SetI2cFreq: Freq set to %d\n", prev_freq );
    
    // Close 
    U2C_CloseDevice( hDevice ); 
    return rc;
}

