/***************************************************************************
 *   Copyright (C)2004 Diolan ( http://www.diolan.com )                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <i2cbridge.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

int main(int argc, char * argv[])
{
	int     rc=0;
	HANDLE  hDevice;
	long    sn;
	unsigned long val;
	U2C_VERSION_INFO fwVersion;
	int i;

	/* Enumerate */
	rc = U2C_GetDeviceCount();
	printf( "U2C_GetDeviceCount: found %d Diolan u2c-12 device(s)\n", rc );
	if( rc == 0 )
		return -1;

	/* Open device '0' */
	hDevice = U2C_OpenDevice( 0 );
	if( hDevice == INVALID_HANDLE_VALUE )
		return -1;
	printf( "U2C_OpenDevice(0): hDevice=%p\n", hDevice );

	/* Get Serial Number */
	rc = U2C_GetSerialNum( hDevice, &sn );
	if( rc ) return rc;
	printf( "U2C_GetSerialNum: %08lx\n", sn );

	/* Get Library Version */
	fwVersion = U2C_GetDllVersion();
	printf( "U2C_GetDllVersion: Major=%02hd Minor=%02hd\n",
		fwVersion.MajorVersion, fwVersion.MinorVersion );

	/* Get FW Version */
	rc = U2C_GetFirmwareVersion( hDevice, &fwVersion);
	if( rc ) return rc;
	printf( "U2C_GetFirmwareVersion: Major=%02hd Minor=%02hd\n",
		fwVersion.MajorVersion, fwVersion.MinorVersion );

	/* Set PORTB bits 0, 1, 2, 3 as output */
	printf("Set PORTB<3, 2, 1, 0> as output ... ");
	rc = U2C_SetIoDirection(hDevice, 0x00000F00U, 0x00000F00U);
	if (rc)
	{
		fprintf(stderr, "ERROR: Failed to set IO direction.\n");
		return rc;
	}
	printf("OK\n");

	for (i = 0; i < 4; i++)
	{
		/* Set PORTB bits 0, 1, 2, 3 to HIGH level */
		printf("Set PORTB<3, 2, 1, 0> to HIGH level ... ");
		rc = U2C_IoWrite(hDevice, 0x00000F00U, 0x00000F00U);
		if (rc)
		{
			fprintf(stderr, "ERROR: Failed to set IO value.\n");
			return rc;
		}
		printf("OK\n");
		sleep(1);

		/* Set PORTB bits 0, 1, 2, 3 to LOW level */
		printf("Set PORTB<3, 2, 1, 0> to LOW level ... ");
		rc = U2C_IoWrite(hDevice, 0x00000000U, 0x00000F00U);
		if (rc)
		{
			fprintf(stderr, "ERROR: Failed to set IO value.\n");
			return rc;
		}
		printf("OK\n");
		sleep(1);
	}

	/* Set/Clear PORTB bits 0, 1, 2, 3 pin-by-pin */
	for (i = 0; i < 4; i++)
	{
		val = (1 << i) << 8;
		printf("Set PORTB pins to %X ... ", (val >> 8) & 0xF);
		rc = U2C_IoWrite(hDevice, val, 0x00000F00U);
		if (rc)
		{
			fprintf(stderr, "ERROR: Failed to set IO value.\n");
			return rc;
		}
		printf("OK\n");
		sleep(1);

		val = ~val;
		printf("Set PORTB pins to %X ... ", (val >> 8) & 0xF);
		rc = U2C_IoWrite(hDevice, val, 0x00000F00U);
		if (rc)
		{
			fprintf(stderr, "ERROR: Failed to set IO value.\n");
			return rc;
		}
		printf("OK\n");
		sleep(1);
	}

	/* Set PORTB bits 0, 1, 2, 3 as input */
	printf("Set PORTB<3, 2, 1, 0> as input ... ");
	rc = U2C_SetIoDirection(hDevice, 0x00000000U, 0x00000F00U);
	if (rc)
	{
		fprintf(stderr, "ERROR: Failed to set IO direction.\n");
		return rc;
	}
	printf("OK\n");


	/* Close */
	U2C_CloseDevice( hDevice );

	return 0;
}

/* End of file */
